﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Kinect;
using System.Diagnostics;
using System.IO;
using NDtw;

namespace BodyTracking
{
    /// <summary>
    /// Lógica de interacción para MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Variables
        KinectSensor _sensor;
        MultiSourceFrameReader _reader;
        IList<Body> _bodies;
        string filename;
        

        bool grabar = false;
        string rightHandState = "-";
        string leftHandState = "-";
        string sign = "-";

        #endregion

        #region Main
        public MainWindow()
        {
            InitializeComponent();
        }
        #endregion

        #region Window
        private void Window_Loaded(object sender, RoutedEventArgs e) //when the window loads we start the sensor
        {
            _sensor = KinectSensor.GetDefault();

            if (_sensor != null)
            {
                _sensor.Open();

                _reader = _sensor.OpenMultiSourceFrameReader(FrameSourceTypes.Body | FrameSourceTypes.Color | FrameSourceTypes.Depth | FrameSourceTypes.Infrared);
                _reader.MultiSourceFrameArrived += Reader_MultiSourceFrameArrived;
            }
        }

        private void Window_Closed(object sender, EventArgs e) //Turn off the sensor
        {
            if (_reader != null)
            {
                _reader.Dispose();
            }

            if (_sensor != null)
            {
                _sensor.Close();
            }
        }
        #endregion
        
        #region Draw

        void DrawLine(Joint first, Joint second)
        {
            if (first.TrackingState != TrackingState.NotTracked || second.TrackingState != TrackingState.NotTracked)
            {
                // 3D space point
                CameraSpacePoint jointPosition1 = first.Position;
                CameraSpacePoint jointPosition2 = second.Position;

                // 2D space point
                Point point1 = new Point();
                Point point2 = new Point();

                ColorSpacePoint colorPoint1 = _sensor.CoordinateMapper.MapCameraPointToColorSpace(jointPosition1);
                ColorSpacePoint colorPoint2 = _sensor.CoordinateMapper.MapCameraPointToColorSpace(jointPosition2);

                point1.X = float.IsInfinity(colorPoint1.X) ? 0 : colorPoint1.X;
                point1.Y = float.IsInfinity(colorPoint1.Y) ? 0 : colorPoint1.Y;
                point2.X = float.IsInfinity(colorPoint2.X) ? 0 : colorPoint2.X;
                point2.Y = float.IsInfinity(colorPoint2.Y) ? 0 : colorPoint2.Y;
                first = first.ScaleTo(canvas.ActualWidth, canvas.ActualHeight);
                second = second.ScaleTo(canvas.ActualWidth, canvas.ActualHeight);

                Line line = new Line
                {
                    X1 = point1.X,
                    Y1 = point1.Y,
                    X2 = point2.X,
                    Y2 = point2.Y,
                    StrokeThickness = 8,
                    Stroke = new SolidColorBrush(Colors.LightBlue)
                };

                canvas.Children.Add(line);
            }
        }

        #endregion
        
        #region BodyTracking
        void Reader_MultiSourceFrameArrived(object sender, MultiSourceFrameArrivedEventArgs e)
        {
            var reference = e.FrameReference.AcquireFrame();

            // Color
            using (var frame = reference.ColorFrameReference.AcquireFrame())
            {
                if (frame != null)
                {
                    
                    {
                        camera.Source = frame.ToBitmap();
                    }
                }
            }

          
            // Body
            using (var frame = reference.BodyFrameReference.AcquireFrame())
            {
                if (frame != null)
                {
                    canvas.Children.Clear();

                    _bodies = new Body[frame.BodyFrameSource.BodyCount];

                    frame.GetAndRefreshBodyData(_bodies);

                    foreach (var body in _bodies)
                    {
                        if (body != null)
                        {
                            if (body.IsTracked)
                            {

                                foreach (Joint joint in body.Joints.Values)
                                {
                                    if (joint.TrackingState != TrackingState.NotTracked)
                                    {
                                        if (joint.Equals(body.Joints[JointType.HipLeft]) || joint.Equals(body.Joints[JointType.HipRight]) || joint.Equals(body.Joints[JointType.KneeLeft])
                                            || joint.Equals(body.Joints[JointType.KneeRight]) || joint.Equals(body.Joints[JointType.AnkleRight]) || joint.Equals(body.Joints[JointType.AnkleLeft]) || joint.Equals(body.Joints[JointType.FootLeft]) || joint.Equals(body.Joints[JointType.FootRight]))
                                            continue;
                                        // 3D space point
                                        CameraSpacePoint jointPosition = joint.Position;

                                        // 2D space point
                                        Point point = new Point();
                                        Point handRightPoint = new Point();
                                        Point handLeftPoint = new Point();

                                        ColorSpacePoint colorPoint = _sensor.CoordinateMapper.MapCameraPointToColorSpace(jointPosition);
                                        ColorSpacePoint handPointRight = _sensor.CoordinateMapper.MapCameraPointToColorSpace(body.Joints[JointType.HandRight].Position);
                                        ColorSpacePoint handPointLeft = _sensor.CoordinateMapper.MapCameraPointToColorSpace(body.Joints[JointType.HandLeft].Position);

                                        point.X = float.IsInfinity(colorPoint.X) ? 0 : colorPoint.X;
                                        point.Y = float.IsInfinity(colorPoint.Y) ? 0 : colorPoint.Y;

                                        handRightPoint.X = float.IsInfinity(handPointRight.X) ? 0 : handPointRight.X;
                                        handRightPoint.Y = float.IsInfinity(handPointRight.Y) ? 0 : handPointRight.Y;

                                        handLeftPoint.X = float.IsInfinity(handPointLeft.X) ? 0 : handPointLeft.X;
                                        handLeftPoint.Y = float.IsInfinity(handPointLeft.Y) ? 0 : handPointLeft.Y;



                                        // Draw
                                        Ellipse ellipse = new Ellipse
                                        {
                                            Fill = Brushes.Green,
                                            Width = 20,
                                            Height = 20
                                        };

                                        Canvas.SetLeft(ellipse, point.X - ellipse.Width / 2);
                                        Canvas.SetTop(ellipse, point.Y - ellipse.Height / 2);

                                        canvas.Children.Add(ellipse);

                                        Ellipse RhandT = new Ellipse
                                        {
                                            Width = 200,
                                            Height = 200,
                                            Stroke = new SolidColorBrush(Colors.Green),
                                            StrokeThickness = 6

                                        };

                                        Ellipse LhandT = new Ellipse
                                        {
                                            Width = 200,
                                            Height = 200,
                                            Stroke = new SolidColorBrush(Colors.Green),
                                            StrokeThickness = 6

                                        };

                                        Ellipse RhandF = new Ellipse
                                        {
                                            Width = 200,
                                            Height = 200,
                                            Stroke = new SolidColorBrush(Colors.Red),
                                            StrokeThickness = 6

                                        };

                                        Ellipse LhandF = new Ellipse
                                        {
                                            Width = 200,
                                            Height = 200,
                                            Stroke = new SolidColorBrush(Colors.Red),
                                            StrokeThickness = 6

                                        };


                                        if (grabar == true)
                                        {
                                            Canvas.SetLeft(RhandT, handRightPoint.X - RhandT.Width / 2);
                                            Canvas.SetTop(RhandT, handRightPoint.Y - RhandT.Height / 2);
                                            Canvas.SetLeft(LhandT, handLeftPoint.X - LhandT.Width / 2);
                                            Canvas.SetTop(LhandT, handLeftPoint.Y - LhandT.Height / 2);

                                            canvas.Children.Add(RhandT);
                                            canvas.Children.Add(LhandT);
                                        }

                                        if (grabar == false)
                                        {
                                            Canvas.SetLeft(RhandF, handRightPoint.X - RhandF.Width / 2);
                                            Canvas.SetTop(RhandF, handRightPoint.Y - RhandF.Height / 2);
                                            Canvas.SetLeft(LhandF, handLeftPoint.X - LhandF.Width / 2);
                                            Canvas.SetTop(LhandF, handLeftPoint.Y - LhandF.Height / 2);

                                            canvas.Children.Add(RhandF);
                                            canvas.Children.Add(LhandF);
                                        }

                                        DrawLine(body.Joints[JointType.Head], body.Joints[JointType.Neck]);
                                        DrawLine(body.Joints[JointType.Neck], body.Joints[JointType.SpineShoulder]);
                                        DrawLine(body.Joints[JointType.SpineShoulder], body.Joints[JointType.ShoulderLeft]);
                                        DrawLine(body.Joints[JointType.SpineShoulder], body.Joints[JointType.ShoulderRight]);
                                        DrawLine(body.Joints[JointType.SpineShoulder], body.Joints[JointType.SpineMid]);
                                        DrawLine(body.Joints[JointType.ShoulderLeft], body.Joints[JointType.ElbowLeft]);
                                        DrawLine(body.Joints[JointType.ShoulderRight], body.Joints[JointType.ElbowRight]);
                                        DrawLine(body.Joints[JointType.ElbowLeft], body.Joints[JointType.WristLeft]);
                                        DrawLine(body.Joints[JointType.ElbowRight], body.Joints[JointType.WristRight]);
                                        DrawLine(body.Joints[JointType.WristLeft], body.Joints[JointType.HandLeft]);
                                        DrawLine(body.Joints[JointType.WristRight], body.Joints[JointType.HandRight]);
                                        DrawLine(body.Joints[JointType.HandLeft], body.Joints[JointType.HandTipLeft]);
                                        DrawLine(body.Joints[JointType.HandRight], body.Joints[JointType.HandTipRight]);
                                        DrawLine(body.Joints[JointType.WristLeft], body.Joints[JointType.ThumbLeft]);
                                        DrawLine(body.Joints[JointType.WristRight], body.Joints[JointType.ThumbRight]);
                                        DrawLine(body.Joints[JointType.SpineMid], body.Joints[JointType.SpineBase]);

                                        
                                            


                                    }
                                }

                                if (body.Joints[JointType.HandRight].Position.Y > body.Joints[JointType.ElbowRight].Position.Y &
                                    body.Joints[JointType.HandLeft].Position.Y > body.Joints[JointType.ElbowLeft].Position.Y)
                                {
                                    if (body.HandLeftState == HandState.Open & body.HandRightState == HandState.Open)
                                    {
                                        grabar = true;
                                    }
                                }

                                if (body.Joints[JointType.HandRight].Position.Y > body.Joints[JointType.ElbowRight].Position.Y &
                                   body.Joints[JointType.HandLeft].Position.Y > body.Joints[JointType.ElbowLeft].Position.Y)
                                {
                                    if (body.HandLeftState == HandState.Closed & body.HandRightState == HandState.Closed)
                                    {
                                        grabar = false;
                                        
                                    }
                                }

                                
                                if (grabar == true)
                                {

                                    _sensor.ReEscale(body.Joints[JointType.HandLeft]);
                                    _sensor.ReEscale(body.Joints[JointType.HandRight]);
                                    _sensor.ReEscale(body.Joints[JointType.HandTipLeft]);
                                    _sensor.ReEscale(body.Joints[JointType.HandTipRight]);
                                    _sensor.ReEscale(body.Joints[JointType.SpineMid]);
                                    _sensor.ReEscale(body.Joints[JointType.Head]);
                                    _sensor.ReEscale(body.Joints[JointType.Neck]);
                                    _sensor.ReEscale(body.Joints[JointType.ElbowLeft]);
                                    _sensor.ReEscale(body.Joints[JointType.ElbowRight]);
                                    _sensor.ReEscale(body.Joints[JointType.SpineShoulder]);
                                    _sensor.ReEscale(body.Joints[JointType.ShoulderLeft]);
                                    _sensor.ReEscale(body.Joints[JointType.ShoulderRight]);
                                    _sensor.ReEscale(body.Joints[JointType.WristLeft]);
                                    _sensor.ReEscale(body.Joints[JointType.WristRight]);


                                    double d1 = Distance(body.Joints[JointType.Head].Position.X, body.Joints[JointType.Head].Position.Y,
                                            body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d2 = Distance(body.Joints[JointType.Neck].Position.X, body.Joints[JointType.Neck].Position.Y,
                                             body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d3 = Distance(body.Joints[JointType.SpineShoulder].Position.X, body.Joints[JointType.SpineShoulder].Position.Y,
                                            body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d4 = Distance(body.Joints[JointType.ShoulderRight].Position.X, body.Joints[JointType.ShoulderRight].Position.Y,
                                           body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d5 = Distance(body.Joints[JointType.ShoulderLeft].Position.X, body.Joints[JointType.ShoulderLeft].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d6 = Distance(body.Joints[JointType.ElbowLeft].Position.X, body.Joints[JointType.ElbowLeft].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d7 = Distance(body.Joints[JointType.ElbowRight].Position.X, body.Joints[JointType.ElbowRight].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d8 = Distance(body.Joints[JointType.WristLeft].Position.X, body.Joints[JointType.WristLeft].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d9 = Distance(body.Joints[JointType.WristRight].Position.X, body.Joints[JointType.WristRight].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d10 = Distance(body.Joints[JointType.HandLeft].Position.X, body.Joints[JointType.HandLeft].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d11 = Distance(body.Joints[JointType.HandRight].Position.X, body.Joints[JointType.HandRight].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d12 = Distance(body.Joints[JointType.HandTipRight].Position.X, body.Joints[JointType.HandTipRight].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);
                                        double d13 = Distance(body.Joints[JointType.HandTipLeft].Position.X, body.Joints[JointType.HandTipLeft].Position.Y,
                                          body.Joints[JointType.SpineMid].Position.X, body.Joints[JointType.SpineMid].Position.Y);


                                    String vector = d1.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) +","
                                        +d2.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d3.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d4.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d5.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d6.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d7.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d8.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d9.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d10.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d11.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d12.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture) + ","
                                        + d13.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture);




                                    
                                   StreamWriter sw = File.AppendText("J:\\" +filename +".txt");
                                    sw.WriteLine(vector);


                                   sw.Close();

                                        

                                    
                                   
                                }

                                switch (body.HandRightState)
                                {
                                    case HandState.Open:
                                        rightHandState = "Open";
                                        break;
                                    case HandState.Closed:
                                        rightHandState = "Closed";
                                        break;
                                    case HandState.Lasso:
                                        rightHandState = "Lasso";
                                        break;
                                    case HandState.Unknown:
                                        rightHandState = "Unknown...";
                                        break;
                                    case HandState.NotTracked:
                                        rightHandState = "Not tracked";
                                        break;
                                    default:
                                        break;
                                }

                                switch (body.HandLeftState)
                                {
                                    case HandState.Open:
                                        leftHandState = "Open";
                                        break;
                                    case HandState.Closed:
                                        leftHandState = "Closed";
                                        break;
                                    case HandState.Lasso:
                                        leftHandState = "Lasso";
                                        break;
                                    case HandState.Unknown:
                                        leftHandState = "Unknown...";
                                        break;
                                    case HandState.NotTracked:
                                        leftHandState = "Not tracked";
                                        break;
                                    default:
                                        break;
                                }

                                rightHand.Text = rightHandState;
                                leftHand.Text = leftHandState;

                            }
                        }
                    }
                }
            }
        }
        #endregion

      

        #region Euclidian_Distance

        public static double Distance(double x1, double y1, double x2, double y2)
        { 
            return Math.Sqrt(((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)));
        }
        #endregion

        #region button

        private void Create(object sender, RoutedEventArgs e) { 


            InputBox.Visibility = System.Windows.Visibility.Visible;
            
        }

        private void YesButton_Click(object sender, RoutedEventArgs e)
        {
            // YesButton Clicked! Let's hide our InputBox and handle the input text.
            InputBox.Visibility = System.Windows.Visibility.Collapsed;

            // Do something with the Input
            filename = InputTextBox.Text;
            FileStream fs = File.Create("J:\\" + filename +".txt");
            

            // Clear InputBox.
            InputTextBox.Text = String.Empty;
        }

        private void NoButton_Click(object sender, RoutedEventArgs e)
        {
            // NoButton Clicked! Let's hide our InputBox.
            InputBox.Visibility = System.Windows.Visibility.Collapsed;

            // Clear InputBox.
            InputTextBox.Text = String.Empty;
        }

        private void Compare(object sender, RoutedEventArgs e)
        {
            
           
            double[] seqA = File.ReadAllLines("J:\\database.txt").SelectMany(s => s.Split(',')).Select(s => double.Parse(s)).ToArray();
            double[] seqB = File.ReadAllLines("J:\\lento.txt").SelectMany(s => s.Split(',')).Select(s => double.Parse(s)).ToArray();

            var distance = new Dtw(seqA, seqB);
            double cost = distance.GetCost();
            signText.Text = cost.ToString("0.00000", System.Globalization.CultureInfo.InvariantCulture);
            
            

        }


        #endregion







    }

}
